unit mainform;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ExtCtrls, ActnMan, ActnCtrls, ActnMenus,
  XPStyleActnCtrls, ActnList, ToolWin, XPMan, ImgList, NLVCore, StdCtrls,
  AboutBox;

type
  TfrmMain = class(TForm)
    stbMain: TStatusBar;
    cbrMain: TCoolBar;
    acmMain: TActionManager;
    ammMain: TActionMainMenuBar;
    pnlMain: TPanel;
    pagMain: TPageControl;
    tabGeneral: TTabSheet;
    tabGranules: TTabSheet;
		tabPermissions: TTabSheet;
    actFileOpen: TAction;
    lvwGranules: TListView;
    lvwPermissions: TListView;
    dlgOpen: TOpenDialog;
    grpLicenseInof: TGroupBox;
    actHelpAbout: TAction;
    actFileExit: TAction;
    lblLicenseNo: TLabel;
    txtLicenseNo: TEdit;
    memLicensee: TMemo;
    lblLicensee: TLabel;
    imlToolbar: TImageList;
    dlgAbout: TAboutBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure actFileOpenExecute(Sender: TObject);
    procedure lvwPermissionsData(Sender: TObject; Item: TListItem);
    procedure lvwGranulesData(Sender: TObject; Item: TListItem);
    procedure actFileExitExecute(Sender: TObject);
    procedure lvwGranulesColumnClick(Sender: TObject; Column: TListColumn);
    procedure actHelpAboutExecute(Sender: TObject);
    procedure lvwGranulesDataFind(Sender: TObject; Find: TItemFind;
      const FindString: String; const FindPosition: TPoint;
      FindData: Pointer; StartIndex: Integer; Direction: TSearchDirection;
      Wrap: Boolean; var Index: Integer);
    procedure lvwPermissionsDataFind(Sender: TObject; Find: TItemFind;
      const FindString: String; const FindPosition: TPoint;
      FindData: Pointer; StartIndex: Integer; Direction: TSearchDirection;
      Wrap: Boolean; var Index: Integer);
  private
		FLicenseFile: TNavisionLicenseFile;
    procedure FormOpenFile(Sender: TObject; FileName: string);
    procedure FormUpdateCaption(Sender: TObject);
    procedure FormUpdateStatus(Sender: TObject);
    procedure WMDropFiles(var Msg: TWMDropFiles); message WM_DROPFILES;
  public
    property LicenseFile: TNavisionLicenseFile read FLicenseFile;
  end;

var
  frmMain: TfrmMain;

implementation

uses
	ShellAPI;

{$R *.dfm}

{ TfrmMain }

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  FLicenseFile := TNavisionLicenseFile.Create;

  if ParamCount > 0 then
  begin
    Show;
    Application.ProcessMessages;
    FormUpdateCaption(Self);
    FormOpenFile(Self, ParamStr(1));
  end
  else
  begin
    FormUpdateCaption(Self);
    FormUpdateStatus(Self);
  end;

  DragAcceptFiles(Handle, true);
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  FLicenseFile.Free;
end;

procedure TfrmMain.FormOpenFile(Sender: TObject; FileName: string);
begin
  Screen.Cursor := crHourglass;
  try
    Application.ProcessMessages;
    LicenseFile.LoadFromFile(FileName);
    lvwGranules.Items.Count := LicenseFile.GranuleCount;
    lvwPermissions.Items.Count := LicenseFile.PermissionRangeCount;
    lvwGranules.Invalidate;
    lvwPermissions.Invalidate;
    txtLicenseNo.Text := LicenseFile.LicenseNo;
    memLicensee.Lines := LicenseFile.Licensee;
    FormUpdateCaption(Self);
    FormUpdateStatus(Self);
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TfrmMain.FormUpdateCaption(Sender: TObject);
resourcestring
  sAppTitle = 'Navision License Viewer';
begin
  with Application do
  begin
    Title := sAppTitle;
    if LicenseFile.FileName <> '' then
      Title := Format('%s - %s', [ExtractFileName(LicenseFile.FileName), Title]);
    Self.Caption := Title;
  end;
end;

procedure TfrmMain.actFileOpenExecute(Sender: TObject);
begin
	if dlgOpen.Execute then
		FormOpenFile(Self, dlgOpen.FileName);
end;

procedure TfrmMain.lvwPermissionsData(Sender: TObject; Item: TListItem);
begin
	with LicenseFile.PermissionRanges[Item.Index] do
	begin
		Item.Caption := ObjectTypeStr;
		Item.SubItems.Add(IntToStr(FromID));
		Item.SubItems.Add(IntToStr(ToID));
		Item.SubItems.Add(PermissionStr);
	end;
end;

procedure TfrmMain.lvwGranulesData(Sender: TObject; Item: TListItem);
begin
  with LicenseFile.Granules[Item.Index] do
  begin
    Item.Caption := IntToStr(ID);
    Item.SubItems.Add(Name);

    case (Amount = 0) of
      true: Item.SubItems.Add('');
      false: Item.SubItems.Add(IntToStr(Amount));
    end;
    
    if Expires <> 0 then
      Item.SubItems.Add(DateToStr(Expires));
  end;
end;

procedure TfrmMain.actFileExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.lvwGranulesColumnClick(Sender: TObject;
  Column: TListColumn);
begin
  LicenseFile.GranulesSortBy := TGranulesSortBy(Column.Index);
  lvwGranules.Invalidate;
end;

procedure TfrmMain.actHelpAboutExecute(Sender: TObject);
begin
	dlgAbout.Execute;
end;

procedure TfrmMain.FormUpdateStatus(Sender: TObject);
resourcestring
  sGranules = '%d granule(s)';
begin
  stbMain.SimpleText := Format(sGranules, [lvwGranules.Items.Count]);
end;

procedure TfrmMain.WMDropFiles(var Msg: TWMDropFiles);
var
  Count: integer;
  FileName: array[0..Max_Path] of char;
begin
  Count := DragQueryFile(Msg.Drop, $FFFFFFFF, FileName, Max_Path);

  if Count > 0 then
  begin
    DragQueryFile(Msg.Drop, 0, FileName, Max_Path);
    FormOpenFile(Self, FileName);
  end;

  DragFinish(Msg.Drop);
end;

procedure TfrmMain.lvwGranulesDataFind(Sender: TObject; Find: TItemFind;
  const FindString: String; const FindPosition: TPoint; FindData: Pointer;
  StartIndex: Integer; Direction: TSearchDirection; Wrap: Boolean;
  var Index: Integer);
var
  i: integer;
  Signature: string;
begin
  for i := StartIndex to LicenseFile.GranuleCount - 1 do
  begin
    Signature := IntToStr(LicenseFile.Granules[i].ID);
    if Copy(Signature, 1, Length(FindString)) = FindString then
		begin
			Index := i;
      exit;
    end;
  end;

  if Wrap and (StartIndex > 0) then
    for i := 0 to StartIndex - 1 do
    begin
      Signature := IntToStr(LicenseFile.Granules[i].ID);
      if Copy(Signature, 1, Length(FindString)) = FindString then
      begin
        Index := i;
        exit;
      end;
    end;

  Index := -1;
end;

procedure TfrmMain.lvwPermissionsDataFind(Sender: TObject; Find: TItemFind;
	const FindString: String; const FindPosition: TPoint; FindData: Pointer;
	StartIndex: Integer; Direction: TSearchDirection; Wrap: Boolean;
	var Index: Integer);
var
	i: integer;
	Signature: string;
begin
	for i := StartIndex to LicenseFile.PermissionRangeCount - 1 do
	begin
		Signature := LicenseFile.PermissionRanges[i].ObjectTypeStr;
		if Lowercase(Copy(Signature, 1, Length(FindString))) = LowerCase(FindString) then
		begin
			Index := i;
			exit;
		end;
	end;

	if Wrap and (StartIndex > 0) then
		for i := 0 to StartIndex - 1 do
		begin
			Signature := LicenseFile.PermissionRanges[i].ObjectTypeStr;
			if LowerCase(Copy(Signature, 1, Length(FindString))) = LowerCase(FindString) then
			begin
				Index := i;
				exit;
			end;
		end;
end;

end.
